/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include <shellapi.h>
#include "application.h"
#include "dialog.h"
#include "splitpath.h"
#include "browser.h"

using namespace plexdata::autoplay;

CApplication::CApplication()
	: m_hInstance(NULL)
	, m_hPrevInstance(NULL)
	, m_nCommandShow(0)
{
	INITCOMMONCONTROLSEX controls = {0};
	controls.dwSize = sizeof(INITCOMMONCONTROLSEX);
	controls.dwICC = ICC_WIN95_CLASSES | ICC_BAR_CLASSES;
	_ASSERT( TRUE == ::InitCommonControlsEx( &controls ) );
}

CApplication::~CApplication()
{
}

bool CApplication::Initialize
	( HINSTANCE hInstance
	, HINSTANCE hPrevInstance
	, LPTSTR    szCommandLine
	, int       nCommandShow 
	)
{
	bool success = false;
	int  nBuffer = MAX_PATH;
	plexdata::string::CString sBuffer;

	this->InstanceHandle( hInstance );
	this->PreviousInstanceHandle( hPrevInstance );
	this->CommandLine( szCommandLine );
	this->CommandShow( nCommandShow );

	while ( true )
	{
		if ( 0 == ::GetModuleFileName( this->m_hInstance, sBuffer.CreateBuffer(nBuffer), nBuffer ) )
		{
			// Unhandled error case!
			break; // End WHILE loop!
		}
		else if ( ERROR_INSUFFICIENT_BUFFER == ::GetLastError() )
		{
			nBuffer *= 2;
			continue; // Typ again with bigger buffer.
		}
		else
		{
			success = true;
			break; // End WHILE loop!
		}
	}

	if ( success )
	{
		// Take over the result!
		sBuffer.CopyBuffer();

		// Save module file name.
		this->m_sModuleFile = sBuffer.String();

		plexdata::string::CSplitPath modSplitter  = this->m_sModuleFile.String();

		// Build and save ini file name.
		modSplitter.Extension( _T(".ini") );
		this->m_sIniFile = modSplitter.Build();

		// Build and save module path.
		modSplitter.Filename( NULL );
		modSplitter.Extension( NULL );
		this->m_sModulePath = modSplitter.Build();

		this->m_cConfig.Load( this->m_sIniFile.String(), IDI_AUTOPLAY );

		plexdata::string::CString sTitle = this->m_cConfig.Title();
		if ( sTitle.IsEmpty() )
		{
			::LoadString
				( this->m_hInstance
				, IDS_AUTOPLAY
				, sTitle.CreateBuffer( 256 )
				, 255 
				);

			sTitle.CopyBuffer();
			this->m_cConfig.Title( sTitle.String() );
		}
	}

	return success;
}

int CApplication::Run()
{
	int     result = -1;
	CDialog dlg;

	if ( dlg.Initialize() )
	{
		result = dlg.Show();
	}
	return result;
}

bool CApplication::ShowHelp( HWND hParent )
{
	bool success = false;

	const int HELP_WIDTH  = 900;
	const int HELP_HEIGHT = 650;

	plexdata::string::CString sURL;
	if ( this->IsLanguageGerman() )
	{
		sURL.Format( _T("res://%s/%d"), this->m_sModuleFile.String(), IDR_HTMLHELP_DE );
	}
	else
	{
		sURL.Format( _T("res://%s/%d"), this->m_sModuleFile.String(), IDR_HTMLHELP_EN );
	}

	RECT rDesktop = {0};
	::GetClientRect( ::GetDesktopWindow(), &rDesktop );

	int nTop  = ( ( rDesktop.bottom - rDesktop.top ) - HELP_HEIGHT ) / 2;
	int nLeft = ( ( rDesktop.right - rDesktop.left ) - HELP_WIDTH  ) / 2;

	plexdata::autoplay::CBrowser theBrowser;
	success = theBrowser.LaunchBrowser
		( hParent
		, sURL
		, m_cConfig.Icon()
		, nTop
		, nLeft
		, HELP_HEIGHT
		, HELP_WIDTH
		);

	if ( !success )
	{
		plexdata::string::CString sMessage;
		plexdata::string::CString sHeader;
		
		::LoadString
			( this->m_hInstance
			, IDS_NOHELPSCREEN
			, sMessage.CreateBuffer( 65 )
			, 64 
			);
		sMessage.CopyBuffer();

		::LoadString
			( this->m_hInstance
			, IDS_ERRORTITLE
			, sHeader.CreateBuffer( 16 )
			, 15 
			);
		sHeader.CopyBuffer();

		::MessageBox( hParent, sMessage.String(), sHeader.String(), MB_OK | MB_ICONERROR );
	}

	return success;
}

bool CApplication::IsLanguageGerman()
{
	TCHAR szLanguage[3] = _T("");

	::LoadString
		( this->m_hInstance
		, IDS_LANGUAGE
		, szLanguage
		, sizeof(szLanguage)/sizeof(TCHAR)
		);
	
	return ( 0 == _tcsicmp( szLanguage, _T("DE") ) );
}

int APIENTRY _tWinMain
	( HINSTANCE hInstance
	, HINSTANCE hPrevInstance
	, LPTSTR    lpCmdLine
	, int       nCmdShow
	)
{
	int result = -1L;

	if ( CApplication::GetInstance().Initialize( hInstance, hPrevInstance, lpCmdLine, nCmdShow ) )
	{
		result = CApplication::GetInstance().Run();
	}

	return result;
}
