/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#pragma once

#include "resource.h"
#include "string.h"
#include "configuration.h"
#include "cursor.h"

namespace plexdata
{
	namespace autoplay
	{
		class CApplication
		{
		private:

			HINSTANCE		          m_hInstance;
			HINSTANCE                 m_hPrevInstance;
			int                       m_nCommandShow;
			CConfiguration            m_cConfig;
			plexdata::string::CString m_sCommandLine;
			plexdata::string::CString m_sModuleFile;
			plexdata::string::CString m_sModulePath;
			plexdata::string::CString m_sIniFile;

		private:

			CApplication();

		public: 

			virtual ~CApplication();

		public:

			static inline CApplication& GetInstance()
			{
				static CApplication theInstance;
				return theInstance;
			}

		public:

			inline void InstanceHandle( HINSTANCE hInstance )
			{
				this->m_hInstance = hInstance;
			}

			inline HINSTANCE InstanceHandle()
			{
				return this->m_hInstance;
			}

			inline void PreviousInstanceHandle( HINSTANCE hPrevInstance )
			{
				this->m_hPrevInstance = hPrevInstance;
			}

			inline HINSTANCE PreviousInstanceHandle()
			{
				return this->m_hPrevInstance;
			}

			inline void CommandLine( LPTSTR szCommandLine )
			{
				this->m_sCommandLine = szCommandLine;
			}

			inline LPCTSTR CommandLine()
			{
				return this->m_sCommandLine.String();
			}

			inline void CommandShow( int nCommandShow )
			{
				this->m_nCommandShow = nCommandShow;
			}

			inline int CommandShow()
			{
				return this->m_nCommandShow;
			}

			inline LPCTSTR ModuleFile()
			{
				return this->m_sModuleFile.String();
			}

			inline LPCTSTR ModulePath()
			{
				return this->m_sModulePath.String();
			}

			inline LPCTSTR IniFile()
			{
				return this->m_sIniFile.String();
			}

			inline CConfiguration& Config()
			{
				return this->m_cConfig;
			}

		public:

			bool Initialize
				( HINSTANCE hInstance
				, HINSTANCE hPrevInstance
				, LPTSTR    szCommandLine
				, int       nCommandShow 
				);

			int Run();

			bool ShowHelp( HWND hParent );

			bool IsLanguageGerman();
		};
	}
}