/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include "browser.h"
#include <atlbase.h>
#include <exdisp.h>

using namespace plexdata::autoplay;
using namespace plexdata::string;

CBrowser::CBrowser()
	: m_hParent(NULL)
	, m_hIcon(NULL)
	, m_nTop(0)
	, m_nLeft(0)
	, m_nHeight(0)
	, m_nWidth(0)
	, m_bResizable(false)
	, m_bToolBar(false)
	, m_bAddressBar(false)
	, m_bMenuBar(false)
	, m_bStatusBar(false)
{
}

CBrowser::~CBrowser()
{
}

bool CBrowser::LaunchBrowser
	( HWND     hParent 
	, CString& sURL
	, HICON    hIcon
	, int      nTop
	, int      nLeft
	, int      nHeight
	, int      nWidth
	, bool     bResizable  /*= false*/
	, bool     bToolBar    /*= false*/
	, bool     bAddressBar /*= false*/
	, bool     bMenuBar    /*= false*/
	, bool     bStatusBar  /*= false*/
	)
{
	bool success = false;

	if ( !sURL.IsEmpty() )
	{
		// Save given parameters.
		this->m_hParent     = hParent;
		this->m_sURL        = sURL;
		this->m_hIcon       = hIcon;
		this->m_nTop        = nTop;
		this->m_nLeft       = nLeft;
		this->m_nHeight     = nHeight;
		this->m_nWidth      = nWidth;
		this->m_bResizable  = bResizable;
		this->m_bToolBar    = bToolBar;
		this->m_bAddressBar = bAddressBar;
		this->m_bMenuBar    = bMenuBar;
		this->m_bStatusBar  = bStatusBar;

		// Start Internet Explorer within a different thread.
		HANDLE hThread = ::CreateThread
			( NULL
			, 0
			, CBrowser::StartBrowser
			, this
			, 0
			, NULL
			);

		if ( NULL != hThread )
		{
			// Wait until Internet Explorer has done its loading operation.
			if ( WAIT_OBJECT_0 == ::WaitForSingleObject( hThread, INFINITE ) )
			{
				DWORD dwExitCode = S_FALSE;

				if ( ::GetExitCodeThread( hThread, &dwExitCode ) )
				{
					success = ( dwExitCode == S_OK ) ? true : false;
				}
			}
		}
	}
	return success;
}

DWORD WINAPI CBrowser::StartBrowser
	( LPVOID pParameter  
	)
{
	HRESULT       nResult    = S_FALSE;
	HRESULT       hResult    = S_FALSE;
	CBrowser*     pThis      = reinterpret_cast<CBrowser*>( pParameter );
	IUnknown*     pUnknown   = NULL;
	IWebBrowser2* pBrowser   = NULL;
	VARIANT       varNothing = {0};

	_ASSERT( pThis != NULL );

	if ( pThis != NULL )
	{
		hResult = ::OleInitialize( NULL );

		if ( SUCCEEDED( hResult ) )
		{
			hResult = ::CoCreateInstance
				( CLSID_InternetExplorer
				, NULL
				, CLSCTX_SERVER
				, IID_IUnknown
				, (LPVOID*)&pUnknown 
				);

			if( SUCCEEDED( hResult ) )
			{
				hResult = pUnknown->QueryInterface
					( IID_IWebBrowser2
					, (LPVOID*)&pBrowser 
					);

				if( SUCCEEDED( hResult ) )
				{
					// Initialize the Internet Explorer properties.
					pBrowser->put_Resizable ( pThis->m_bResizable  ? VARIANT_TRUE : VARIANT_FALSE );
					pBrowser->put_ToolBar   ( pThis->m_bToolBar    ? TRUE         : FALSE         ); // Integer!
					pBrowser->put_AddressBar( pThis->m_bAddressBar ? VARIANT_TRUE : VARIANT_FALSE );
					pBrowser->put_MenuBar   ( pThis->m_bMenuBar    ? VARIANT_TRUE : VARIANT_FALSE );
					pBrowser->put_StatusBar ( pThis->m_bStatusBar  ? VARIANT_TRUE : VARIANT_FALSE );
					pBrowser->put_Top       ( pThis->m_nTop    );
					pBrowser->put_Left      ( pThis->m_nLeft   );
					pBrowser->put_Width     ( pThis->m_nWidth  );
					pBrowser->put_Height    ( pThis->m_nHeight );

					// Start browser navigation.
					hResult = pBrowser->Navigate
						( CComBSTR( pThis->m_sURL.String() )
						, &varNothing
						, &varNothing
						, &varNothing
						, &varNothing
						);

					if( SUCCEEDED( hResult ) )
					{
						// Browser window should always be visible.
						pBrowser->put_Visible( VARIANT_TRUE );

						// Wait until the Internet Explorer is not longer busy.
						VARIANT_BOOL bBusy = VARIANT_TRUE;
						while ( bBusy == VARIANT_TRUE )
						{
							::Sleep( 500 );
							pBrowser->get_Busy( &bBusy );
						}

						// Set more Internet Explorer properties.
						HWND hWnd = NULL;
						pBrowser->get_HWND( (SHANDLE_PTR*)&hWnd );
						if( ::IsWindow( hWnd ) && pThis->m_hIcon != NULL)
						{
							::SendMessage
								( hWnd
								, WM_SETICON
								, (WPARAM)ICON_SMALL
								, (LPARAM)pThis->m_hIcon 
								);

							::SetForegroundWindow( hWnd );
						}

						// Set execution result.
						nResult = S_OK;
					}
					pBrowser->Release();
				}
				pUnknown->Release();
			}
			::OleUninitialize();
		}
	}
	return nResult;
}
