/*
 * Copyright (C) 2010  Axel Kesseler
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include "stdafx.h"
#include "string.h"
#include <stdarg.h>

using namespace plexdata::string;

CString::CString()
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( _T("") );
}

CString::CString( const TCHAR* szString )
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( szString );
}

CString::CString( CString& rString )
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( rString );
}

CString::CString( CString* pString )
	: m_nString(0)
	, m_szString(NULL)
	, m_szBuffer(NULL)
{
	this->Copy( pString );
}

CString::~CString()
{
	this->Destroy();
	this->ReleaseBuffer();
}

bool CString::Append( const TCHAR* szString )
{
	if ( szString != NULL )
	{
		CString sTemp = *this;
		return this->Format( _T("%s%s"), sTemp.m_szString, szString );
	}
	else
	{
		return false;
	}
}

bool CString::Append( CString& rString )
{
	CString sTemp = *this;
	return this->Format( _T("%s%s"), sTemp.m_szString, rString.m_szString );
}

bool CString::Append( CString* pString )
{
	if ( pString != NULL )
	{
		CString sTemp = *this;
		return this->Format( _T("%s%s"), sTemp.m_szString, pString->m_szString );
	}
	else
	{
		return false;
	}
}

bool CString::Copy( const TCHAR* szString )
{
	return this->Create( szString );
}

bool CString::Copy( CString& rString )
{
	return this->Create( rString.m_szString );
}

bool CString::Copy( CString* pString )
{
	if ( pString != NULL )
	{
		return this->Create( pString->m_szString );
	}
	else
	{
		return false;
	}
}

bool CString::Equals( const TCHAR* szString, bool bIgnore/*=true*/ )
{
	if ( szString != NULL )
	{
		if ( bIgnore )
		{
			return ( 0 == _tcsicmp( this->m_szString, szString ) );
		}
		else
		{
			return ( 0 == _tcscmp( this->m_szString, szString ) );
		}
	}
	else
	{
		return false;
	}
}

bool CString::Equals( CString& rString, bool bIgnore/*=true*/ )
{
	if ( rString.m_szString != NULL )
	{
		if ( bIgnore )
		{
			return ( 0 == _tcsicmp( this->m_szString, rString.m_szString ) );
		}
		else
		{
			return ( 0 == _tcscmp( this->m_szString, rString.m_szString ) );
		}
	}
	else
	{
		return false;
	}
}

bool CString::Equals( CString* pString, bool bIgnore/*=true*/ )
{
	if ( pString != NULL && pString->m_szString != NULL )
	{
		if ( bIgnore )
		{
			return ( 0 == _tcsicmp( this->m_szString, pString->m_szString ) );
		}
		else
		{
			return ( 0 == _tcscmp( this->m_szString, pString->m_szString ) );
		}
	}
	else
	{
		return false;
	}
}

bool CString::Format( const TCHAR* szFormat, ... )
{
	bool    success = false;
	long    length  = 0;
	double  dummy   = 0.0; // Load floating point library, otherwise runtime error R6002 will occur!
	va_list argList;

	if ( szFormat != NULL )
	{
		va_start( argList, szFormat );

		 // _vsctprintf doesn't include terminating '\0'!
		length = _vsctprintf( szFormat, argList ) + 1;

		if ( length > 0 && this->Create( length ) )
		{
			 // _vstprintf_s doesn't include terminating '\0'!
			success = ( 0 < _vstprintf_s( this->m_szString, this->m_nString, szFormat, argList ) );
			this->m_nString = _tcslen( this->m_szString ); // BUGFIX: String length must be recalculated.
		}

		va_end( argList );
	}

	return success;
}

CString CString::Substring( long nFirst, long nLast/*=-1L*/ )
{
	CString result;

	if ( this->m_szString != NULL )
	{
		if ( nLast == -1L )
		{
			nLast = this->m_nString; // Sub-string is up to the end.
		}
		else if ( nLast > this->m_nString )
		{
			nLast = this->m_nString; // Limit result string if needed.
		}

		if ( nFirst <= nLast )
		{
			long nCount = nLast - nFirst;

			if ( nFirst < this->m_nString && ( nFirst + nCount ) <= this->m_nString )
			{
				if ( result.Create( nCount ) )
				{
					_tcsncpy_s( result.m_szString, result.m_nString + 1, &this->m_szString[nFirst], nCount );
				}
			}
		}
	}
	return result;
}

CString CString::Trim()
{
	return this->LeftTrim().RightTrim();
}

CString CString::LeftTrim()
{
	CString result;

	if ( this->m_szString != NULL )
	{
		for ( long index=0; index < this->m_nString; index++ )
		{
			TCHAR cCurrent = this->m_szString[index];
			if ( cCurrent != _T(' ') && cCurrent != _T('\t') )
			{
				result =  this->Substring( index );
				break;
			}
		}
	}
	return result;
}

CString CString::RightTrim()
{
	CString result;

	if ( this->m_szString != NULL )
	{
		// Try to (reverse) find last character.
		for ( long index=this->m_nString; index > 0; index-- )
		{
			TCHAR cCurrent = this->m_szString[index];
			if ( cCurrent != _T(' ') && cCurrent != _T('\t') && cCurrent != _T('\0') )
			{
				result = this->Substring( 0, index + 1 );
				break;
			}
		}
	}
	return result;
}

long CString::FindFirst( const TCHAR cFind )
{
	long result = -1L;

	if ( this->m_szString != NULL )
	{
		// Try to find first character.
		for ( long index=0; index < this->m_nString; index++ )
		{
			if ( this->m_szString[index] == cFind )
			{
				result = index;
				break; // End FOR loop!
			}
		}
	}
	return result;
}

long CString::FindNext( const TCHAR cFind, long nStart )
{
	bool success = false;
	long nNext   = -1L;

	if ( this->m_szString != NULL && 0 <= nStart && nStart < this->m_nString )
	{
		// Try to find next character just behind given position.
		for ( long index=nStart; index < this->m_nString; index++ )
		{
			if ( this->m_szString[index] == cFind )
			{
				nNext = index; // Save new position and finish.
				break; // End FOR loop!
			}
		}
	}
	return nNext;
}

long CString::FindLast( const TCHAR cFind )
{
	long result = -1L;

	if ( this->m_szString != NULL )
	{
		// Try to (reverse) find last character.
		for ( long index=this->m_nString; index > 0; index-- )
		{
			if ( this->m_szString[index] == cFind )
			{
				result = index;
				break; // End FOR loop!
			}
		}
	}
	return result;
}

TCHAR CString::GetAt( long index )
{
	TCHAR result = _T('\0');

	if ( this->m_szString != NULL )
	{
		if ( 0 <= index && index < this->m_nString )
		{
			result = this->m_szString[index];
		}
	}
	return result;
}

TCHAR* CString::CreateBuffer( long nCount )
{
	this->ReleaseBuffer();

	this->m_szBuffer = new TCHAR[ nCount + 1 ];

	if ( this->m_szBuffer != NULL )
	{
		for ( long index=0; index < nCount + 1; index++ )
		{
			this->m_szBuffer[index] = _T('\0');
		}
	}
	return this->m_szBuffer;
}

bool CString::CopyBuffer()
{
	return this->Create( this->m_szBuffer );
}

void CString::ReleaseBuffer()
{
	delete this->m_szBuffer;
	this->m_szBuffer = NULL;
}

bool CString::Create( long nCount )
{
	bool success = false;

	this->Destroy();

	this->m_nString  = nCount;
	this->m_szString = new TCHAR[ this->m_nString + 1 ];

	if ( this->m_szString != NULL )
	{
		for ( long index=0; index < this->m_nString + 1; index++ )
		{
			this->m_szString[index] = _T('\0');
		}
		success = true;
	}
	else
	{
		this->m_nString = 0;
	}
	return success;
}

bool CString::Create( const TCHAR* szString )
{
	bool success = false;

	this->Destroy();

	if ( szString != NULL )
	{
		this->m_nString  = _tcslen( szString );
		this->m_szString = new TCHAR[ this->m_nString + 1 ];

		if ( this->m_szString != NULL )
		{
			if ( 0 == _tcscpy_s( this->m_szString, this->m_nString + 1, szString ) )
			{
				success = true;
			}
		}
		else
		{
			this->m_nString = 0;
		}
	}
	return success;
}

void CString::Destroy()
{
	delete this->m_szString;

	this->m_nString  = 0;
	this->m_szString = NULL;
}
